@@ -8,7 +8,6 @@ import (
 	"syscall"
 	"time"
 
-	log "github.com/Sirupsen/logrus"
 	"github.com/containernetworking/cni/pkg/ns"
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
@@ -21,6 +20,7 @@ import (
 	"github.com/projectcalico/libcalico-go/lib/logutils"
 	cnet "github.com/projectcalico/libcalico-go/lib/net"
 	"github.com/projectcalico/libcalico-go/lib/testutils"
+	log "github.com/sirupsen/logrus"
 	"github.com/vishvananda/netlink"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/client-go/kubernetes"
@@ -33,6 +33,8 @@ func init() {
 	rand.Seed(time.Now().UTC().UnixNano())
 	log.SetFormatter(&logutils.Formatter{})
 	log.AddHook(&logutils.ContextHook{})
+	log.SetOutput(GinkgoWriter)
+	log.SetLevel(log.DebugLevel)
 }
 
 var _ = Describe("CalicoCni", func() {
@@ -44,7 +46,6 @@ var _ = Describe("CalicoCni", func() {
 
 	Describe("Run Calico CNI plugin in K8s mode", func() {
 		utils.ConfigureLogging("info")
-		logger := utils.CreateContextLogger("k8s_tests")
 		cniVersion := os.Getenv("CNI_SPEC_VERSION")
 
 		Context("using host-local IPAM", func() {
@@ -92,7 +93,7 @@ var _ = Describe("CalicoCni", func() {
 				if err != nil {
 					panic(err)
 				}
-				containerID, netnspath, session, contVeth, contAddresses, contRoutes, _, err := CreateContainer(netconf, name, "")
+				containerID, session, contVeth, contAddresses, contRoutes, contNs, err := CreateContainer(netconf, name, "")
 
 				Expect(err).ShouldNot(HaveOccurred())
 				Eventually(session).Should(gexec.Exit())
@@ -182,7 +183,7 @@ var _ = Describe("CalicoCni", func() {
 						Type:      syscall.RTN_UNICAST,
 					})))
 
-				_, err = DeleteContainer(netconf, netnspath, name)
+				_, err = DeleteContainer(netconf, contNs.Path(), name)
 				Expect(err).ShouldNot(HaveOccurred())
 
 				// Make sure there are no endpoints anymore
@@ -191,7 +192,7 @@ var _ = Describe("CalicoCni", func() {
 				Expect(endpoints.Items).Should(HaveLen(0))
 
 				// Make sure the interface has been removed from the namespace
-				targetNs, _ := ns.GetNS(netnspath)
+				targetNs, _ := ns.GetNS(contNs.Path())
 				err = targetNs.Do(func(_ ns.NetNS) error {
 					_, err = netlink.LinkByName("eth0")
 					return err
@@ -228,11 +229,11 @@ var _ = Describe("CalicoCni", func() {
 					if err := CreateHostVeth("", name, K8S_TEST_NS); err != nil {
 						panic(err)
 					}
-					_, netnspath, session, _, _, _, _, err := CreateContainer(netconf, name, "")
+					_, session, _, _, _, contNs, err := CreateContainer(netconf, name, "")
 					Expect(err).ShouldNot(HaveOccurred())
 					Eventually(session).Should(gexec.Exit(0))
 
-					_, err = DeleteContainer(netconf, netnspath, name)
+					_, err = DeleteContainer(netconf, contNs.Path(), name)
 					Expect(err).ShouldNot(HaveOccurred())
 				})
 			})
@@ -284,18 +285,18 @@ var _ = Describe("CalicoCni", func() {
 					})
 					Expect(err).NotTo(HaveOccurred())
 
-					logger.Infof("Created POD object: %v", pod)
+					log.Infof("Created POD object: %v", pod)
 
-					_, netnspath, _, _, contAddresses, _, _, err := CreateContainer(netconfCalicoIPAM, name, "")
+					_, _, _, contAddresses, _, contNs, err := CreateContainer(netconfCalicoIPAM, name, "")
 					Expect(err).NotTo(HaveOccurred())
 
 					podIP := contAddresses[0].IP
-					logger.Infof("All container IPs: %v", contAddresses)
-					logger.Infof("Container got IP address: %s", podIP)
+					log.Infof("All container IPs: %v", contAddresses)
+					log.Infof("Container got IP address: %s", podIP)
 					Expect(ipPoolCIDR.Contains(podIP)).To(BeTrue())
 
 					// Delete the container.
-					_, err = DeleteContainer(netconfCalicoIPAM, netnspath, name)
+					_, err = DeleteContainer(netconfCalicoIPAM, contNs.Path(), name)
 					Expect(err).ShouldNot(HaveOccurred())
 				})
 			})
@@ -340,15 +341,15 @@ var _ = Describe("CalicoCni", func() {
 					})
 					Expect(err).NotTo(HaveOccurred())
 
-					logger.Infof("Created POD object: %v", pod)
+					log.Infof("Created POD object: %v", pod)
 
-					containerID, netnspath, _, contVeth, contAddresses, _, _, err := CreateContainer(netconfCalicoIPAM, name, "")
+					containerID, _, contVeth, contAddresses, _, contNs, err := CreateContainer(netconfCalicoIPAM, name, "")
 					Expect(err).NotTo(HaveOccurred())
 					mac := contVeth.Attrs().HardwareAddr
 
 					podIP := contAddresses[0].IP
-					logger.Infof("All container IPs: %v", contAddresses)
-					logger.Infof("Container got IP address: %s", podIP)
+					log.Infof("All container IPs: %v", contAddresses)
+					log.Infof("Container got IP address: %s", podIP)
 					Expect(podIP).Should(Equal(assignIP))
 
 					interfaceName := k8s.VethNameForWorkload(fmt.Sprintf("%s.%s", K8S_TEST_NS, name))
@@ -379,7 +380,7 @@ var _ = Describe("CalicoCni", func() {
 					}))
 
 					// Delete the container.
-					_, err = DeleteContainer(netconfCalicoIPAM, netnspath, name)
+					_, err = DeleteContainer(netconfCalicoIPAM, contNs.Path(), name)
 					Expect(err).ShouldNot(HaveOccurred())
 				})
 			})
@@ -436,15 +437,15 @@ var _ = Describe("CalicoCni", func() {
 					})
 					Expect(err).NotTo(HaveOccurred())
 
-					logger.Infof("Created POD object: %v", pod)
+					log.Infof("Created POD object: %v", pod)
 
-					containerID, netnspath, _, contVeth, contAddresses, _, _, err := CreateContainer(netconfCalicoIPAM, name, "")
+					containerID, _, contVeth, contAddresses, _, contNt, err := CreateContainer(netconfCalicoIPAM, name, "")
 					Expect(err).NotTo(HaveOccurred())
 					mac := contVeth.Attrs().HardwareAddr
 
 					podIP := contAddresses[0].IP
-					logger.Infof("All container IPs: %v", contAddresses)
-					logger.Infof("Container got IP address: %s", podIP)
+					log.Infof("All container IPs: %v", contAddresses)
+					log.Infof("Container got IP address: %s", podIP)
 					Expect(podIP).Should(Equal(assignIP))
 
 					interfaceName := k8s.VethNameForWorkload(fmt.Sprintf("%s.%s", K8S_TEST_NS, name))
@@ -475,7 +476,7 @@ var _ = Describe("CalicoCni", func() {
 					}))
 
 					// Delete the container.
-					_, err = DeleteContainer(netconfCalicoIPAM, netnspath, name)
+					_, err = DeleteContainer(netconfCalicoIPAM, contNt.Path(), name)
 					Expect(err).ShouldNot(HaveOccurred())
 				})
 			})
@@ -530,15 +531,15 @@ var _ = Describe("CalicoCni", func() {
 					})
 					Expect(err).NotTo(HaveOccurred())
 
-					logger.Infof("Created POD object: %v", pod)
+					log.Infof("Created POD object: %v", pod)
 
-					containerID, netnspath, _, contVeth, contAddresses, _, _, err := CreateContainer(netconfCalicoIPAM, name, "")
+					containerID, _, contVeth, contAddresses, _, contNs, err := CreateContainer(netconfCalicoIPAM, name, "")
 					Expect(err).NotTo(HaveOccurred())
 					mac := contVeth.Attrs().HardwareAddr
 
 					podIP := contAddresses[0].IP
-					logger.Infof("All container IPs: %v", contAddresses)
-					logger.Infof("Container got IP address: %s", podIP)
+					log.Infof("All container IPs: %v", contAddresses)
+					log.Infof("Container got IP address: %s", podIP)
 					Expect(podIP).Should(Equal(assignIP))
 
 					interfaceName := k8s.VethNameForWorkload(fmt.Sprintf("%s.%s", K8S_TEST_NS, name))
@@ -569,13 +570,13 @@ var _ = Describe("CalicoCni", func() {
 					}))
 
 					// Delete the container.
-					_, err = DeleteContainer(netconfCalicoIPAM, netnspath, name)
+					_, err = DeleteContainer(netconfCalicoIPAM, contNs.Path(), name)
 					Expect(err).ShouldNot(HaveOccurred())
 				})
 			})
 
 			Context("Using host-local IPAM: request an IP then release it, and then request it again", func() {
-				It("Should successfully assign IP both times and successfully release it in the middle", func() {
+				It("should successfully assign IP both times and successfully release it in the middle", func() {
 					netconfHostLocalIPAM := fmt.Sprintf(`
 				  {
 					"cniVersion": "%s",
@@ -624,15 +625,15 @@ var _ = Describe("CalicoCni", func() {
 					requestedIP := "10.0.0.42"
 					expectedIP := net.IPv4(10, 0, 0, 42).To4()
 
-					_, netnspath, _, _, contAddresses, _, _, err := CreateContainer(netconfHostLocalIPAM, name, requestedIP)
+					_, _, _, contAddresses, _, contNs, err := CreateContainer(netconfHostLocalIPAM, name, requestedIP)
 					Expect(err).NotTo(HaveOccurred())
 
 					podIP := contAddresses[0].IP
-					logger.Infof("All container IPs: %v", contAddresses)
+					log.Infof("All container IPs: %v", contAddresses)
 					Expect(podIP).Should(Equal(expectedIP))
 
 					By("Deleting the pod we created earlier")
-					_, err = DeleteContainer(netconfHostLocalIPAM, netnspath, name)
+					_, err = DeleteContainer(netconfHostLocalIPAM, contNs.Path(), name)
 					Expect(err).ShouldNot(HaveOccurred())
 
 					By("Creating a second pod with the same IP address as the first pod")
@@ -648,11 +649,11 @@ var _ = Describe("CalicoCni", func() {
 					})
 					Expect(err).NotTo(HaveOccurred())
 
-					_, netnspath, _, _, contAddresses, _, _, err = CreateContainer(netconfHostLocalIPAM, name2, requestedIP)
+					_, _, _, contAddresses, _, contNs, err = CreateContainer(netconfHostLocalIPAM, name2, requestedIP)
 					Expect(err).NotTo(HaveOccurred())
 
 					pod2IP := contAddresses[0].IP
-					logger.Infof("All container IPs: %v", contAddresses)
+					log.Infof("All container IPs: %v", contAddresses)
 					Expect(pod2IP).Should(Equal(expectedIP))
 				})
 			})
@@ -706,10 +707,10 @@ var _ = Describe("CalicoCni", func() {
 
 					Expect(err).NotTo(HaveOccurred())
 
-					logger.Infof("Created POD object: %v", pod)
+					log.Infof("Created POD object: %v", pod)
 
 					// ADD the container with passing a CNI_ContainerID "X".
-					_, netnspath, session, _, _, _, _, err := CreateContainerWithId(netconf, name, "", cniContainerIDX)
+					_, session, _, _, _, contNs, err := CreateContainerWithId(netconf, name, "", cniContainerIDX)
 					Expect(err).ShouldNot(HaveOccurred())
 					Eventually(session).Should(gexec.Exit())
 
@@ -737,7 +738,7 @@ var _ = Describe("CalicoCni", func() {
 					}))
 
 					// Delete the container with the CNI_ContainerID "X".
-					exitCode, err := DeleteContainerWithId(netconf, netnspath, name, cniContainerIDX)
+					exitCode, err := DeleteContainerWithId(netconf, contNs.Path(), name, cniContainerIDX)
 					Expect(err).ShouldNot(HaveOccurred())
 					Expect(exitCode).Should(Equal(0))
 
@@ -747,7 +748,7 @@ var _ = Describe("CalicoCni", func() {
 					Expect(endpoints.Items).Should(HaveLen(0))
 
 					// ADD a new container with passing a CNI_ContainerID "Y".
-					_, netnspath, session, _, _, _, _, err = CreateContainerWithId(netconf, name, "", cniContainerIDY)
+					_, session, _, _, _, contNs, err = CreateContainerWithId(netconf, name, "", cniContainerIDY)
 					Expect(err).ShouldNot(HaveOccurred())
 					Eventually(session).Should(gexec.Exit())
 
@@ -775,7 +776,7 @@ var _ = Describe("CalicoCni", func() {
 					}))
 
 					// Delete the container with the CNI_ContainerID "X" again.
-					exitCode, err = DeleteContainerWithId(netconf, netnspath, name, cniContainerIDX)
+					exitCode, err = DeleteContainerWithId(netconf, contNs.Path(), name, cniContainerIDX)
 					Expect(err).ShouldNot(HaveOccurred())
 					Expect(exitCode).Should(Equal(0))
 
@@ -798,6 +799,145 @@ var _ = Describe("CalicoCni", func() {
 				})
 			})
 
+			// This specific test case is for an issue where k8s would send two ADDs with different container IDs for the same Pod.
+			Context("Multiple ADD calls with different container IDs followed by DEL calls for stale container IDs", func() {
+				It("Use different CNI_ContainerIDs to ADD and DEL the container", func() {
+					netconf := fmt.Sprintf(`
+				{
+			      "cniVersion": "%s",
+				  "name": "net7",
+				  "type": "calico",
+				  "etcd_endpoints": "http://%s:2379",
+			 	  "ipam": {
+			    	 "type": "calico-ipam"
+			         },
+					"kubernetes": {
+					  "k8s_api_root": "http://127.0.0.1:8080"
+					 },
+					"policy": {"type": "k8s"},
+					"log_level":"info"
+				}`, cniVersion, os.Getenv("ETCD_IP"))
+
+					// Create a new ipPool.
+					ipPool := "10.0.0.0/24"
+					c, _ := client.NewFromEnv()
+					testutils.CreateNewIPPool(*c, ipPool, false, false, true)
+
+					config, err := clientcmd.DefaultClientConfig.ClientConfig()
+					Expect(err).NotTo(HaveOccurred())
+
+					clientset, err := kubernetes.NewForConfig(config)
+					Expect(err).NotTo(HaveOccurred())
+
+					// Now create a K8s pod.
+					name := fmt.Sprintf("run%d-pool", rand.Uint32())
+
+					cniContainerIDX := "container-id-00X"
+					cniContainerIDY := "container-id-00Y"
+
+					pod, err := clientset.Pods(K8S_TEST_NS).Create(
+						&v1.Pod{
+							ObjectMeta: metav1.ObjectMeta{
+								Name: name,
+							},
+							Spec: v1.PodSpec{Containers: []v1.Container{{
+								Name:  fmt.Sprintf("container-%s", name),
+								Image: "ignore",
+							}}},
+						})
+
+					Expect(err).NotTo(HaveOccurred())
+
+					log.Infof("Created POD object: %v", pod)
+
+					// ADD the container with passing a CNI_CONTAINERID of "X".
+					_, session, _, _, _, contNs, err := CreateContainerWithId(netconf, name, "", cniContainerIDX)
+					Expect(err).ShouldNot(HaveOccurred())
+					Eventually(session).Should(gexec.Exit())
+
+					result, err := GetResultForCurrent(session, cniVersion)
+					if err != nil {
+						log.Fatalf("Error getting result from the session: %v\n", err)
+					}
+					log.Printf("Unmarshaled result: %v\n", result)
+
+					// Assert that the endpoint is created in the backend datastore with ContainerID "X".
+					endpoints, err := calicoClient.WorkloadEndpoints().List(api.WorkloadEndpointMetadata{})
+					Expect(err).ShouldNot(HaveOccurred())
+					Expect(endpoints.Items).Should(HaveLen(1))
+
+					// Set the Revision to nil since we can't assert it's exact value.
+					endpoints.Items[0].Metadata.Revision = nil
+					Expect(endpoints.Items[0].Metadata).Should(Equal(api.WorkloadEndpointMetadata{
+						Node:             hostname,
+						Name:             "eth0",
+						Workload:         fmt.Sprintf("test.%s", name),
+						ActiveInstanceID: cniContainerIDX,
+						Orchestrator:     "k8s",
+						Labels:           map[string]string{"calico/k8s_ns": "test"},
+					}))
+
+					// ADD the container with passing a CNI_CONTAINERID of "Y"
+					_, session, _, _, _, contNs, err = CreateContainerWithId(netconf, name, "", cniContainerIDY)
+					Expect(err).ShouldNot(HaveOccurred())
+					Eventually(session).Should(gexec.Exit())
+
+					result, err = GetResultForCurrent(session, cniVersion)
+					if err != nil {
+						log.Fatalf("Error getting result from the session: %v\n", err)
+					}
+					log.Printf("Unmarshaled result: %v\n", result)
+
+					// Assert that the endpoint is created in the backend datastore with ContainerID "Y".
+					endpoints, err = calicoClient.WorkloadEndpoints().List(api.WorkloadEndpointMetadata{})
+					Expect(err).ShouldNot(HaveOccurred())
+					Expect(endpoints.Items).Should(HaveLen(1))
+
+					// Set the Revision to nil since we can't assert it's exact value.
+					endpoints.Items[0].Metadata.Revision = nil
+					Expect(endpoints.Items[0].Metadata).Should(Equal(api.WorkloadEndpointMetadata{
+						Node:             hostname,
+						Name:             "eth0",
+						Workload:         fmt.Sprintf("test.%s", name),
+						ActiveInstanceID: cniContainerIDY,
+						Orchestrator:     "k8s",
+						Labels:           map[string]string{"calico/k8s_ns": "test"},
+					}))
+
+					// Delete the container with the CNI_CONTAINERID "X".
+					exitCode, err := DeleteContainerWithId(netconf, contNs.Path(), name, cniContainerIDX)
+					Expect(err).ShouldNot(HaveOccurred())
+					Expect(exitCode).Should(Equal(0))
+
+					// Assert that the endpoint in the backend datastore still has ContainerID "Y".
+					endpoints, err = calicoClient.WorkloadEndpoints().List(api.WorkloadEndpointMetadata{})
+					Expect(err).ShouldNot(HaveOccurred())
+					Expect(endpoints.Items).Should(HaveLen(1))
+
+					// Set the Revision to nil since we can't assert it's exact value.
+					endpoints.Items[0].Metadata.Revision = nil
+					Expect(endpoints.Items[0].Metadata).Should(Equal(api.WorkloadEndpointMetadata{
+						Node:             hostname,
+						Name:             "eth0",
+						Workload:         fmt.Sprintf("test.%s", name),
+						ActiveInstanceID: cniContainerIDY,
+						Orchestrator:     "k8s",
+						Labels:           map[string]string{"calico/k8s_ns": "test"},
+					}))
+
+					// Delete the container with the CNI_CONTAINERID "Y".
+					exitCode, err = DeleteContainerWithId(netconf, contNs.Path(), name, cniContainerIDY)
+					Expect(err).ShouldNot(HaveOccurred())
+					Expect(exitCode).Should(Equal(0))
+
+					// Assert that the endpoint in the backend datastore is now gone.
+					endpoints, err = calicoClient.WorkloadEndpoints().List(api.WorkloadEndpointMetadata{})
+					Expect(err).ShouldNot(HaveOccurred())
+					Expect(endpoints.Items).Should(HaveLen(0))
+
+				})
+			})
+
 			Context("Create a container then send another ADD for the same container", func() {
 				netconf := fmt.Sprintf(`
 				{
@@ -842,9 +982,9 @@ var _ = Describe("CalicoCni", func() {
 
 					Expect(err).NotTo(HaveOccurred())
 
-					logger.Infof("Created POD object: %v", pod)
+					log.Infof("Created POD object: %v", pod)
 
-					containerID, netnspath, session, _, _, _, contNs, err := CreateContainer(netconf, name, "")
+					containerID, session, _, _, _, contNs, err := CreateContainer(netconf, name, "")
 					Expect(err).ShouldNot(HaveOccurred())
 					Eventually(session).Should(gexec.Exit())
 
@@ -872,7 +1012,7 @@ var _ = Describe("CalicoCni", func() {
 					}))
 
 					// Try to create the same container (so CNI receives the ADD for the same endpoint again)
-					session, _, _, _, err = RunCNIPluginWithId(netconf, name, "", netnspath, containerID, contNs)
+					session, _, _, _, err = RunCNIPluginWithId(netconf, name, "", containerID, contNs)
 					Expect(err).ShouldNot(HaveOccurred())
 					Eventually(session).Should(gexec.Exit())
 
@@ -884,7 +1024,7 @@ var _ = Describe("CalicoCni", func() {
 					log.Printf("Unmarshalled result from second ADD: %v\n", resultSecondAdd)
 					Expect(resultSecondAdd).Should(Equal(result))
 
-					_, err = DeleteContainer(netconf, netnspath, "")
+					_, err = DeleteContainer(netconf, contNs.Path(), "")
 					Expect(err).ShouldNot(HaveOccurred())
 				})
 			})
@@ -927,9 +1067,9 @@ var _ = Describe("CalicoCni", func() {
 					})
 					Expect(err).NotTo(HaveOccurred())
 
-					logger.Infof("Created POD object: %v", pod)
+					log.Infof("Created POD object: %v", pod)
 
-					_, netnspath, _, _, _, _, _, err := CreateContainer(netconfCalicoIPAM, name, "")
+					_, _, _, _, _, contNs, err := CreateContainer(netconfCalicoIPAM, name, "")
 					Expect(err).To(HaveOccurred())
 
 					// Make sure the WorkloadEndpoint is not created in the datastore.
@@ -938,7 +1078,7 @@ var _ = Describe("CalicoCni", func() {
 					Expect(endpoints.Items).Should(HaveLen(0))
 
 					// Delete the container.
-					_, err = DeleteContainer(netconfCalicoIPAM, netnspath, name)
+					_, err = DeleteContainer(netconfCalicoIPAM, contNs.Path(), name)
 					Expect(err).ShouldNot(HaveOccurred())
 				})
 			})
